@@ -204,12 +204,6 @@ void dd_move_x(struct gmx_domdec_t* dd, const matrix box, gmx::ArrayRef<gmx::RVe
  */
 void dd_move_f(struct gmx_domdec_t* dd, gmx::ForceWithShiftForces* forceWithShiftForces, gmx_wallcycle* wcycle);
 
-/*! \brief Communicate a real for each atom to the neighboring cells. */
-void dd_atom_spread_real(struct gmx_domdec_t* dd, real v[]);
-
-/*! \brief Sum the contributions to a real for each atom over the neighboring cells. */
-void dd_atom_sum_real(struct gmx_domdec_t* dd, real v[]);
-
 /*! \brief Reset all the statistics and counters for total run counting */
 void reset_dd_statistics_counters(struct gmx_domdec_t* dd);
 
@@ -229,9 +223,12 @@ void dd_move_x_constraints(struct gmx_domdec_t*     dd,
                            bool                     bX1IsCoord);
 
 /*! \brief Communicates the coordinates involved in virtual sites */
-void dd_move_x_vsites(const gmx_domdec_t& dd, const matrix box, rvec* x);
+void dd_move_x_vsites(const gmx_domdec_t& dd, const matrix box, gmx::ArrayRef<gmx::RVec> x);
 /*! \brief Communicates the positions and velocities involved in virtual sites */
-void dd_move_x_and_v_vsites(const gmx_domdec_t& dd, const matrix box, rvec* x, rvec* v);
+void dd_move_x_and_v_vsites(const gmx_domdec_t&      dd,
+                            const matrix             box,
+                            gmx::ArrayRef<gmx::RVec> x,
+                            gmx::ArrayRef<gmx::RVec> v);
 
 /*! \brief Returns the local atom count array for all constraints
  *
@@ -290,7 +287,7 @@ void communicateGpuHaloForces(const t_commrec&
 /*! \brief Wraps the \c positions so that atoms from the same
  * update group share the same periodic image wrt \c box.
  *
- * When DD and update groups are in use, the simulation master rank
+ * When DD and update groups are in use, the simulation main rank
  * should call this to ensure that e.g. when restarting a simulation
  * that did not use update groups that the coordinates satisfy the new
  * requirements.
