@@ -61,6 +61,7 @@
 #include "gromacs/math/utilities.h"
 #include "gromacs/math/vec.h"
 #include "gromacs/mdlib/perf_est.h"
+#include "gromacs/mdrunutility/mdmodulesnotifiers.h"
 #include "gromacs/mdtypes/commrec.h"
 #include "gromacs/mdtypes/inputrec.h"
 #include "gromacs/mdtypes/md_enums.h"
@@ -68,9 +69,9 @@
 #include "gromacs/topology/topology.h"
 #include "gromacs/utility/fatalerror.h"
 #include "gromacs/utility/logger.h"
-#include "gromacs/utility/mdmodulesnotifiers.h"
 #include "gromacs/utility/stringutil.h"
 
+#include "atomdistribution.h"
 #include "box.h"
 #include "domdec_internal.h"
 #include "utility.h"
@@ -298,7 +299,7 @@ real comm_box_frac(const gmx::IVec& dd_nc, real cutoff, const gmx_ddbox_t& ddbox
 /*! \brief Return whether the DD inhomogeneous in the z direction */
 static gmx_bool inhomogeneous_z(const t_inputrec& ir)
 {
-    return ((EEL_PME(ir.coulombtype) || ir.coulombtype == CoulombInteractionType::Ewald)
+    return ((usingPme(ir.coulombtype) || ir.coulombtype == CoulombInteractionType::Ewald)
             && ir.pbcType == PbcType::Xyz && ir.ewald_geometry == EwaldGeometry::ThreeDC);
 }
 
@@ -362,7 +363,8 @@ static float comm_cost_est(real               limit,
         for (int j = i + 1; j < ddbox.npbcdim; j++)
         {
             if (box[j][i] != 0 || ir.deform[j][i] != 0
-                || (ir.epc != PressureCoupling::No && ir.compress[j][i] != 0))
+                || (ir.pressureCouplingOptions.epc != PressureCoupling::No
+                    && ir.pressureCouplingOptions.compress[j][i] != 0))
             {
                 if (nc[j] > 1 && nc[i] == 1)
                 {
@@ -411,7 +413,7 @@ static float comm_cost_est(real               limit,
         }
     }
 
-    if (EEL_PME(ir.coulombtype) || EVDW_PME(ir.vdwtype))
+    if (usingPme(ir.coulombtype) || usingLJPme(ir.vdwtype))
     {
         /* Check the PME grid restrictions.
          * Currently these can only be invalid here with too few grid lines
@@ -646,7 +648,7 @@ static gmx::IVec optimizeDDCells(const gmx::MDLogger& mdlog,
     // which is the number of PP ranks when not using separate
     // PME-only ranks.
     const int numRanksDoingPmeWork =
-            (EEL_PME(ir.coulombtype) ? ((numPmeOnlyRanks > 0) ? numPmeOnlyRanks : numPPRanks) : 0);
+            (usingPme(ir.coulombtype) ? ((numPmeOnlyRanks > 0) ? numPmeOnlyRanks : numPPRanks) : 0);
 
     if (systemInfo.haveInterDomainBondeds)
     {
@@ -738,7 +740,7 @@ real getDDGridSetupCellSizeLimit(const gmx::MDLogger& mdlog,
                         "Scaling the initial minimum size with 1/%g (option -dds) = %g", dlb_scale, 1 / dlb_scale);
         cellSizeLimit /= dlb_scale;
     }
-    else if (ir.epc != PressureCoupling::No)
+    else if (ir.pressureCouplingOptions.epc != PressureCoupling::No)
     {
         GMX_LOG(mdlog.info)
                 .appendTextFormatted(
@@ -873,7 +875,7 @@ static int getNumPmeOnlyRanksToUse(const gmx::MDLogger&                  mdlog,
 {
     int numPmeOnlyRanks = 0;
 
-    if (!(EEL_PME(ir.coulombtype) || EVDW_PME(ir.vdwtype)))
+    if (!(usingPme(ir.coulombtype) || usingLJPme(ir.vdwtype)))
     {
         // System does not use PME for Electrostatics or LJ
         numPmeOnlyRanks = 0;
@@ -980,16 +982,16 @@ DDGridSetup getDDGridSetup(const gmx::MDLogger&                  mdlog,
     {
         set_ddbox_cr(ddRole, communicator, nullptr, ir, box, xGlobal, ddbox);
 
-        if (ddRole == DDRole::Master)
+        if (ddRole == DDRole::Main)
         {
             numDomains = optimizeDDCells(
                     mdlog, numRanksRequested, numPmeOnlyRanks, cellSizeLimit, mtop, box, *ddbox, ir, systemInfo);
         }
     }
 
-    /* Communicate the information set by the master to all ranks */
+    /* Communicate the information set by the coordinator to all ranks */
     gmx_bcast(sizeof(numDomains), numDomains, communicator);
-    if (EEL_PME(ir.coulombtype))
+    if (usingPme(ir.coulombtype))
     {
         gmx_bcast(sizeof(numPmeOnlyRanks), &numPmeOnlyRanks, communicator);
     }
