@@ -58,6 +58,7 @@
 #include "gromacs/topology/idef.h"
 #include "gromacs/topology/ifunc.h"
 #include "gromacs/topology/mtop_util.h"
+#include "gromacs/topology/topology.h"
 #include "gromacs/utility/arrayref.h"
 #include "gromacs/utility/fatalerror.h"
 #include "gromacs/utility/logger.h"
@@ -179,7 +180,7 @@ static std::string printMissingInteractionsMolblock(const t_commrec*         cr,
             int j     = mol * nril_mol + j_mol;
             if (isAssigned[j] == 0 && !(interaction_function[ftype].flags & IF_VSITE))
             {
-                if (DDMASTER(cr->dd))
+                if (DDMAIN(cr->dd))
                 {
                     if (i == 0)
                     {
@@ -280,7 +281,7 @@ static void printMissingInteractionsAtoms(const MDLogger&               mdlog,
 
     gmx_sumi(F_NRE, cl, cr);
 
-    if (DDMASTER(dd))
+    if (DDMAIN(dd))
     {
         GMX_LOG(mdlog.warning).appendText("A list of missing interactions:");
         int rest_global = expectedNumGlobalBondedInteractions;
@@ -328,7 +329,7 @@ static void printMissingInteractionsAtoms(const MDLogger&               mdlog,
     {
         errorMessage =
                 "One or more interactions were assigned to multiple domains of the domain "
-                "decompostion. Please report this bug.";
+                "decomposition. Please report this bug.";
     }
     else
     {
@@ -342,7 +343,7 @@ static void printMissingInteractionsAtoms(const MDLogger&               mdlog,
                 dd_cutoff_multibody(dd),
                 dd_cutoff_twobody(dd));
     }
-    gmx_fatal_collective(FARGS, cr->mpi_comm_mygroup, MASTER(cr), "%s", errorMessage.c_str());
+    gmx_fatal_collective(FARGS, cr->mpi_comm_mygroup, MAIN(cr), "%s", errorMessage.c_str());
 }
 
 /*! \brief Data to help check local topology construction
