@@ -157,7 +157,7 @@ static void print_cg_move(FILE*               fplog,
                           rvec                cm_new,
                           real                pos_d)
 {
-    const gmx_domdec_comm_t* comm = dd->comm;
+    const gmx_domdec_comm_t* comm = dd->comm.get();
     std::string              mesg;
 
     fprintf(fplog, "\nStep %" PRId64 ":\n", step);
@@ -558,7 +558,7 @@ void dd_redistribute_cg(FILE*         fplog,
                         t_nrnb*       nrnb,
                         int*          ncg_moved)
 {
-    gmx_domdec_comm_t* comm = dd->comm;
+    gmx_domdec_comm_t* comm = dd->comm.get();
 
     if (dd->unitCellInfo.haveScrewPBC)
     {
@@ -753,7 +753,7 @@ void dd_redistribute_cg(FILE*         fplog,
 
     int* moved = getMovedBuffer(comm, 0, dd->numHomeAtoms);
 
-    clear_and_mark_ind(move, dd->globalAtomIndices, dd->ga2la, moved);
+    clear_and_mark_ind(move, dd->globalAtomIndices, dd->ga2la.get(), moved);
 
     /* Now we can remove the excess global atom-group indices from the list */
     dd->globalAtomGroupIndices.resize(dd->numHomeAtoms);
@@ -782,7 +782,11 @@ void dd_redistribute_cg(FILE*         fplog,
                 fprintf(debug, "Sending ddim %d dir %d: nat %d\n", d, dir, nat[cdd]);
             }
             int atomCountToReceive;
-            ddSendrecv(dd, d, dir, &nat[cdd], 1, &atomCountToReceive, 1);
+            ddSendrecv(dd,
+                       d,
+                       dir,
+                       gmx::arrayRefFromArray(&nat[cdd], 1),
+                       gmx::arrayRefFromArray(&atomCountToReceive, 1));
 
             flagBuffer.resize((totalAtomsReceived + atomCountToReceive) * DD_CGIBS);
 
@@ -790,10 +794,9 @@ void dd_redistribute_cg(FILE*         fplog,
             ddSendrecv(dd,
                        d,
                        dir,
-                       comm->cggl_flag[cdd].data(),
-                       nat[cdd] * DD_CGIBS,
-                       flagBuffer.buffer.data() + totalAtomsReceived * DD_CGIBS,
-                       atomCountToReceive * DD_CGIBS);
+                       gmx::arrayRefFromArray(comm->cggl_flag[cdd].data(), nat[cdd] * DD_CGIBS),
+                       gmx::arrayRefFromArray(flagBuffer.buffer.data() + totalAtomsReceived * DD_CGIBS,
+                                              atomCountToReceive * DD_CGIBS));
 
             const int nvs = nat[cdd] * (1 + nvec);
             const int i   = atomCountToReceive * (1 + nvec);
@@ -803,10 +806,8 @@ void dd_redistribute_cg(FILE*         fplog,
             ddSendrecv(dd,
                        d,
                        dir,
-                       as_rvec_array(comm->cgcm_state[cdd].data()),
-                       nvs,
-                       as_rvec_array(rvecBuffer.buffer.data()) + nvr,
-                       i);
+                       gmx::arrayRefFromArray(comm->cgcm_state[cdd].data(), nvs),
+                       gmx::arrayRefFromArray(rvecBuffer.buffer.data() + nvr, i));
             totalAtomsReceived += atomCountToReceive;
             nvr += i;
         }
@@ -841,7 +842,7 @@ void dd_redistribute_cg(FILE*         fplog,
                 /* Check which direction this cg should go */
                 for (int d2 = d + 1; (d2 < dd->ndim && mc == -1); d2++)
                 {
-                    if (isDlbOn(dd->comm))
+                    if (isDlbOn(dd->comm->dlbState))
                     {
                         /* The cell boundaries for dimension d2 are not equal
                          * for each cell row of the lower dimension(s),
