@@ -43,6 +43,7 @@
 
 #include "gromacs/fileio/gmxfio.h"
 #include "gromacs/topology/atoms.h"
+#include "gromacs/topology/mtop_atomloops.h"
 #include "gromacs/topology/mtop_util.h"
 #include "gromacs/topology/symtab.h"
 #include "gromacs/topology/topology.h"
@@ -65,7 +66,7 @@ static void get_coordnum_fp(FILE* in, char* title, int* natoms)
     }
 }
 
-void get_coordnum(const char* infile, int* natoms)
+void get_coordnum(const std::filesystem::path& infile, int* natoms)
 {
     FILE* in;
     char  title[STRLEN];
@@ -80,15 +81,15 @@ void get_coordnum(const char* infile, int* natoms)
  * We have removed writing of variable precision to avoid compatibility
  * issues with other software packages.
  */
-static gmx_bool get_w_conf(FILE*       in,
-                           const char* infile,
-                           char*       title,
-                           t_symtab*   symtab,
-                           t_atoms*    atoms,
-                           int*        ndec,
-                           rvec        x[],
-                           rvec*       v,
-                           matrix      box)
+static gmx_bool get_w_conf(FILE*                        in,
+                           const std::filesystem::path& infile,
+                           char*                        title,
+                           t_symtab*                    symtab,
+                           t_atoms*                     atoms,
+                           int*                         ndec,
+                           rvec                         x[],
+                           rvec*                        v,
+                           matrix                       box)
 {
     char     name[6];
     char     resname[6], oldresname[6];
@@ -139,11 +140,11 @@ static gmx_bool get_w_conf(FILE*       in,
     {
         if ((fgets2(line, STRLEN, in)) == nullptr)
         {
-            gmx_fatal(FARGS, "Unexpected end of file in file %s at line %d", infile, i + 2);
+            gmx_fatal(FARGS, "Unexpected end of file in file %s at line %d", infile.u8string().c_str(), i + 2);
         }
         if (strlen(line) < 39)
         {
-            gmx_fatal(FARGS, "Invalid line in %s for atom %d:\n%s", infile, i + 1, line);
+            gmx_fatal(FARGS, "Invalid line in %s for atom %d:\n%s", infile.u8string().c_str(), i + 1, line);
         }
 
         /* determine read precision from distance between periods
@@ -154,12 +155,12 @@ static gmx_bool get_w_conf(FILE*       in,
             p1     = strchr(line, '.');
             if (p1 == nullptr)
             {
-                gmx_fatal(FARGS, "A coordinate in file %s does not contain a '.'", infile);
+                gmx_fatal(FARGS, "A coordinate in file %s does not contain a '.'", infile.u8string().c_str());
             }
             p2 = strchr(&p1[1], '.');
             if (p2 == nullptr)
             {
-                gmx_fatal(FARGS, "A coordinate in file %s does not contain a '.'", infile);
+                gmx_fatal(FARGS, "A coordinate in file %s does not contain a '.'", infile.u8string().c_str());
             }
             ddist = p2 - p1;
             *ndec = ddist - 5;
@@ -167,7 +168,7 @@ static gmx_bool get_w_conf(FILE*       in,
             p3 = strchr(&p2[1], '.');
             if (p3 == nullptr)
             {
-                gmx_fatal(FARGS, "A coordinate in file %s does not contain a '.'", infile);
+                gmx_fatal(FARGS, "A coordinate in file %s does not contain a '.'", infile.u8string().c_str());
             }
 
             if (p3 - p2 != ddist)
@@ -175,7 +176,7 @@ static gmx_bool get_w_conf(FILE*       in,
                 gmx_fatal(FARGS,
                           "The spacing of the decimal points in file %s is not consistent for x, y "
                           "and z",
-                          infile);
+                          infile.u8string().c_str());
             }
         }
 
@@ -192,7 +193,7 @@ static gmx_bool get_w_conf(FILE*       in,
             newres++;
             if (newres >= natoms)
             {
-                gmx_fatal(FARGS, "More residues than atoms in %s (natoms = %d)", infile, natoms);
+                gmx_fatal(FARGS, "More residues than atoms in %s (natoms = %d)", infile.u8string().c_str(), natoms);
             }
             atoms->atom[i].resind = newres;
             t_atoms_set_resinfo(atoms, i, symtab, resname, resnr, ' ', 0, ' ');
@@ -227,7 +228,7 @@ static gmx_bool get_w_conf(FILE*       in,
                 gmx_fatal(FARGS,
                           "Something is wrong in the coordinate formatting of file %s. Note that "
                           "gro is fixed format (see the manual)",
-                          infile);
+                          infile.u8string().c_str());
             }
             else
             {
@@ -265,7 +266,7 @@ static gmx_bool get_w_conf(FILE*       in,
     fgets2(line, STRLEN, in);
     if (sscanf(line, "%lf%lf%lf", &x1, &y1, &z1) != 3)
     {
-        gmx_warning("Bad box in file %s", infile);
+        gmx_warning("Bad box in file %s", infile.u8string().c_str());
 
         /* Generate a cubic box */
         for (m = 0; (m < DIM); m++)
@@ -314,7 +315,13 @@ static gmx_bool get_w_conf(FILE*       in,
     return bVel;
 }
 
-void gmx_gro_read_conf(const char* infile, t_symtab* symtab, char** name, t_atoms* atoms, rvec x[], rvec* v, matrix box)
+void gmx_gro_read_conf(const std::filesystem::path& infile,
+                       t_symtab*                    symtab,
+                       char**                       name,
+                       t_atoms*                     atoms,
+                       rvec                         x[],
+                       rvec*                        v,
+                       matrix                       box)
 {
     FILE* in = gmx_fio_fopen(infile, "r");
     int   ndec;
@@ -359,7 +366,7 @@ gmx_bool gro_next_x_or_v(FILE* status, t_trxframe* fr)
     snew(atoms.resinfo, fr->natoms);
     snew(atoms.atomname, fr->natoms);
 
-    fr->bV    = get_w_conf(status, title, title, &symtab, &atoms, &ndec, fr->x, fr->v, fr->box);
+    fr->bV = get_w_conf(status, std::string(title), title, &symtab, &atoms, &ndec, fr->x, fr->v, fr->box);
     fr->bPrec = TRUE;
     fr->prec  = 1;
     /* prec = 10^ndec: */
@@ -570,12 +577,12 @@ void write_hconf_p(FILE* out, const char* title, const t_atoms* atoms, const rve
     sfree(aa);
 }
 
-void write_conf_p(const char*    outfile,
-                  const char*    title,
-                  const t_atoms* atoms,
-                  const rvec*    x,
-                  const rvec*    v,
-                  const matrix   box)
+void write_conf_p(const std::filesystem::path& outfile,
+                  const char*                  title,
+                  const t_atoms*               atoms,
+                  const rvec*                  x,
+                  const rvec*                  v,
+                  const matrix                 box)
 {
     FILE* out;
 
