@@ -262,13 +262,13 @@ static gmx_bool do_trr_frame(t_fileio* fio,
  *
  ************************************************************/
 
-void gmx_trr_read_single_header(const char* fn, gmx_trr_header_t* header)
+void gmx_trr_read_single_header(const std::filesystem::path& fn, gmx_trr_header_t* header)
 {
     t_fileio* fio = gmx_trr_open(fn, "r");
     gmx_bool  bOK;
     if (!do_trr_frame_header(fio, true, header, &bOK))
     {
-        gmx_fatal(FARGS, "Empty file %s", fn);
+        gmx_fatal(FARGS, "Empty file %s", fn.u8string().c_str());
     }
     gmx_trr_close(fio);
 }
@@ -278,15 +278,15 @@ gmx_bool gmx_trr_read_frame_header(t_fileio* fio, gmx_trr_header_t* header, gmx_
     return do_trr_frame_header(fio, true, header, bOK);
 }
 
-void gmx_trr_write_single_frame(const char* fn,
-                                int64_t     step,
-                                real        t,
-                                real        lambda,
-                                const rvec* box,
-                                int         natoms,
-                                const rvec* x,
-                                const rvec* v,
-                                const rvec* f)
+void gmx_trr_write_single_frame(const std::filesystem::path& fn,
+                                int64_t                      step,
+                                real                         t,
+                                real                         lambda,
+                                const rvec*                  box,
+                                int                          natoms,
+                                const rvec*                  x,
+                                const rvec*                  v,
+                                const rvec*                  f)
 {
     t_fileio* fio = gmx_trr_open(fn, "w");
     do_trr_frame(fio,
@@ -302,15 +302,15 @@ void gmx_trr_write_single_frame(const char* fn,
     gmx_trr_close(fio);
 }
 
-void gmx_trr_read_single_frame(const char* fn,
-                               int64_t*    step,
-                               real*       t,
-                               real*       lambda,
-                               rvec*       box,
-                               int*        natoms,
-                               rvec*       x,
-                               rvec*       v,
-                               rvec*       f)
+void gmx_trr_read_single_frame(const std::filesystem::path& fn,
+                               int64_t*                     step,
+                               real*                        t,
+                               real*                        lambda,
+                               rvec*                        box,
+                               int*                         natoms,
+                               rvec*                        x,
+                               rvec*                        v,
+                               rvec*                        f)
 {
     t_fileio* fio = gmx_trr_open(fn, "r");
     do_trr_frame(fio, true, step, t, lambda, box, natoms, x, v, f);
@@ -361,7 +361,7 @@ gmx_bool gmx_trr_read_frame_data(t_fileio* fio, gmx_trr_header_t* header, rvec*
     return do_trr_frame_data(fio, header, box, x, v, f);
 }
 
-t_fileio* gmx_trr_open(const char* fn, const char* mode)
+t_fileio* gmx_trr_open(const std::filesystem::path& fn, const char* mode)
 {
     return gmx_fio_open(fn, mode);
 }
