@@ -267,57 +267,58 @@ gmx_bool read_next_vmd_frame(gmx_vmdplugin_t* vmdplugin, t_trxframe* fr)
     return true;
 }
 
-static int load_vmd_library(const char* fn, gmx_vmdplugin_t* vmdplugin)
+static int load_vmd_library(const std::filesystem::path& fn, gmx_vmdplugin_t* vmdplugin)
 {
     const char* err;
     int         ret = 0;
 #if !GMX_NATIVE_WINDOWS
-    glob_t            globbuf;
-    const std::string defpath_suffix = "/plugins/*/molfile";
-    const std::string defpathenv     = GMX_VMD_PLUGIN_PATH;
+    glob_t                      globbuf;
+    const std::string           defpath_suffix = "/plugins/*/molfile";
+    const std::filesystem::path defpathenv     = GMX_VMD_PLUGIN_PATH;
 #else
-    WIN32_FIND_DATA ffd;
-    HANDLE          hFind = INVALID_HANDLE_VALUE;
-    char            progfolder[GMX_PATH_MAX];
-    std::string     defpath_suffix = "\\plugins\\WIN32\\molfile";
+    WIN32_FIND_DATA       ffd;
+    HANDLE                hFind = INVALID_HANDLE_VALUE;
+    char                  progfolder[GMX_PATH_MAX];
+    std::filesystem::path defpath_suffix = "\\plugins\\WIN32\\molfile";
     SHGetFolderPath(NULL, CSIDL_PROGRAM_FILES, NULL, SHGFP_TYPE_CURRENT, progfolder);
-    std::string defpathenv =
-            gmx::formatString("%s\\University of Illinois\\VMD\\plugins\\WIN32\\molfile", progfolder);
+    std::filesystem::path defpathenv =
+            progfolder / "University of Illinois" / "VMD" / "plugins" / "WIN32" / "molfile";
 #endif
 
     vmdplugin->api      = nullptr;
-    vmdplugin->filetype = strrchr(fn, '.');
-    if (!vmdplugin->filetype)
+    vmdplugin->filetype = fn.extension();
+    if (!fn.has_extension())
     {
         return 0;
     }
-    vmdplugin->filetype++;
+    vmdplugin->filetype = vmdplugin->filetype.string().substr(1);
 
     /* First look for an explicit path given at run time for the
      * plugins, then an implicit run-time path, and finally for one
      * given at configure time. This last might be hard-coded to the
      * default for VMD installs. */
-    const char* pathenv = getenv("VMD_PLUGIN_PATH");
-    std::string fallBackPathEnv;
-    if (!pathenv)
+    const char*           pathEnvChar = getenv("VMD_PLUGIN_PATH");
+    std::filesystem::path pathenv     = pathEnvChar != nullptr ? pathEnvChar : "";
+    std::filesystem::path fallBackPathEnv;
+    if (pathenv.empty())
     {
         pathenv = getenv("VMDDIR");
-        if (!pathenv)
+        if (pathenv.empty())
         {
             printf("\nNeither VMD_PLUGIN_PATH or VMDDIR set. ");
             printf("Using default location:\n%s\n", defpathenv.c_str());
-            pathenv = defpathenv.c_str();
+            pathenv = defpathenv;
         }
         else
         {
             printf("\nVMD_PLUGIN_PATH no set, but VMDDIR is set. ");
-            fallBackPathEnv = gmx::Path::join(pathenv, defpath_suffix);
-            pathenv         = fallBackPathEnv.c_str();
-            printf("Using semi-default location:\n%s\n", pathenv);
+            fallBackPathEnv = pathenv / defpath_suffix;
+            pathenv         = fallBackPathEnv;
+            printf("Using semi-default location:\n%s\n", pathenv.c_str());
         }
     }
 #if !GMX_NATIVE_WINDOWS
-    std::string pathname = gmx::Path::join(pathenv, "/*.so");
+    auto pathname = pathenv / "/*.so";
     glob(pathname.c_str(), 0, nullptr, &globbuf);
     if (globbuf.gl_pathc == 0)
     {
@@ -338,8 +339,8 @@ static int load_vmd_library(const char* fn, gmx_vmdplugin_t* vmdplugin)
     }
     globfree(&globbuf);
 #else
-    std::string pathname = gmx::Path::join(pathenv, "\\*.so");
-    hFind                = FindFirstFile(pathname.c_str(), &ffd);
+    auto pathname = pathenv / "*.so";
+    hFind         = FindFirstFile(pathname.c_str(), &ffd);
     if (INVALID_HANDLE_VALUE == hFind)
     {
         printf("\nNo VMD Plugins found\n");
@@ -347,7 +348,7 @@ static int load_vmd_library(const char* fn, gmx_vmdplugin_t* vmdplugin)
     }
     do
     {
-        std::string filename = gmx::Path::join(pathenv, ffd.cFileName);
+        auto filename = pathenv / ffd.cFileName;
         ret |= load_sharedlibrary_plugins(filename.c_str(), vmdplugin);
     } while (FindNextFile(hFind, &ffd) != 0 && vmdplugin->api == NULL);
     FindClose(hFind);
@@ -370,7 +371,7 @@ static int load_vmd_library(const char* fn, gmx_vmdplugin_t* vmdplugin)
 
     if (vmdplugin->api == nullptr)
     {
-        printf("\nNo plugin for %s found\n", vmdplugin->filetype);
+        printf("\nNo plugin for %s found\n", vmdplugin->filetype.c_str());
         return 0;
     }
 
@@ -385,7 +386,7 @@ static int load_vmd_library(const char* fn, gmx_vmdplugin_t* vmdplugin)
     return 1;
 }
 
-int read_first_vmd_frame(const char* fn, gmx_vmdplugin_t** vmdpluginp, t_trxframe* fr)
+int read_first_vmd_frame(const std::filesystem::path& fn, gmx_vmdplugin_t** vmdpluginp, t_trxframe* fr)
 {
     molfile_timestep_metadata_t* metadata = nullptr;
     gmx_vmdplugin_t*             vmdplugin;
@@ -397,27 +398,31 @@ int read_first_vmd_frame(const char* fn, gmx_vmdplugin_t** vmdpluginp, t_trxfram
         return 0;
     }
 
-    vmdplugin->handle = vmdplugin->api->open_file_read(fn, vmdplugin->filetype, &fr->natoms);
+    vmdplugin->handle =
+            vmdplugin->api->open_file_read(fn.c_str(), vmdplugin->filetype.c_str(), &fr->natoms);
 
     if (!vmdplugin->handle)
     {
-        fprintf(stderr, "\nError: could not open file '%s' for reading.\n", fn);
+        fprintf(stderr, "\nError: could not open file '%s' for reading.\n", fn.c_str());
         return 0;
     }
 
     if (fr->natoms == MOLFILE_NUMATOMS_UNKNOWN)
     {
-        fprintf(stderr, "\nFormat of file %s does not record number of atoms.\n", fn);
+        fprintf(stderr, "\nFormat of file %s does not record number of atoms.\n", fn.c_str());
         return 0;
     }
     else if (fr->natoms == MOLFILE_NUMATOMS_NONE)
     {
-        fprintf(stderr, "\nNo atoms found by VMD plugin in file %s.\n", fn);
+        fprintf(stderr, "\nNo atoms found by VMD plugin in file %s.\n", fn.c_str());
         return 0;
     }
     else if (fr->natoms < 1) /*should not be reached*/
     {
-        fprintf(stderr, "\nUnknown number of atoms %d for VMD plugin opening file %s.\n", fr->natoms, fn);
+        fprintf(stderr,
+                "\nUnknown number of atoms %d for VMD plugin opening file %s.\n",
+                fr->natoms,
+                fn.c_str());
         return 0;
     }
 
