@@ -56,14 +56,8 @@
 /* The source code in this file should be thread-safe.
       Please keep it that way. */
 
-CommrecHandle init_commrec(MPI_Comm communicator)
+std::unique_ptr<t_commrec> init_commrec(MPI_Comm communicator)
 {
-    CommrecHandle handle;
-    t_commrec*    cr;
-
-    snew(cr, 1);
-    handle.reset(cr);
-
     int rankInCommunicator, sizeOfCommunicator;
 #if GMX_MPI
 #    if GMX_LIB_MPI
@@ -77,6 +71,8 @@ CommrecHandle init_commrec(MPI_Comm communicator)
     sizeOfCommunicator = 1;
 #endif
 
+    std::unique_ptr<t_commrec> cr = std::make_unique<t_commrec>();
+
     cr->mpiDefaultCommunicator    = communicator;
     cr->sizeOfDefaultCommunicator = sizeOfCommunicator;
     cr->rankInDefaultCommunicator = rankInCommunicator;
@@ -93,36 +89,7 @@ CommrecHandle init_commrec(MPI_Comm communicator)
     // TODO cr->duty should not be initialized here
     cr->duty = (DUTY_PP | DUTY_PME);
 
-    return handle;
-}
-
-void done_commrec(t_commrec* cr)
-{
-    if (MASTER(cr))
-    {
-        if (nullptr != cr->dd)
-        {
-            // TODO: implement
-            // done_domdec(cr->dd);
-        }
-    }
-#if GMX_MPI
-    // TODO We need to be able to free communicators, but the
-    // structure of the commrec and domdec initialization code makes
-    // it hard to avoid both leaks and double frees.
-    bool mySimIsMyGroup = (cr->mpi_comm_mysim == cr->mpi_comm_mygroup);
-    if (cr->mpi_comm_mysim != MPI_COMM_NULL && cr->mpi_comm_mysim != MPI_COMM_WORLD)
-    {
-        // TODO see above
-        // MPI_Comm_free(&cr->mpi_comm_mysim);
-    }
-    if (!mySimIsMyGroup && cr->mpi_comm_mygroup != MPI_COMM_NULL && cr->mpi_comm_mygroup != MPI_COMM_WORLD)
-    {
-        // TODO see above
-        // MPI_Comm_free(&cr->mpi_comm_mygroup);
-    }
-#endif
-    sfree(cr);
+    return cr;
 }
 
 void gmx_setup_nodecomm(FILE gmx_unused* fplog, t_commrec* cr)
@@ -336,16 +303,16 @@ void gmx_sumi(int gmx_unused nr, int gmx_unused r[], const t_commrec gmx_unused*
 #endif
 }
 
-const char* opt2fn_master(const char* opt, int nfile, const t_filenm fnm[], t_commrec* cr)
+const char* opt2fn_main(const char* opt, int nfile, const t_filenm fnm[], t_commrec* cr)
 {
-    return SIMMASTER(cr) ? opt2fn(opt, nfile, fnm) : nullptr;
+    return SIMMAIN(cr) ? opt2fn(opt, nfile, fnm) : nullptr;
 }
 
 void gmx_fatal_collective(int                    f_errno,
                           const char*            file,
                           int                    line,
                           MPI_Comm               comm,
-                          gmx_bool               bMaster,
+                          gmx_bool               bMain,
                           gmx_fmtstr const char* fmt,
                           ...)
 {
@@ -363,6 +330,6 @@ void gmx_fatal_collective(int                    f_errno,
 #endif
 
     va_start(ap, fmt);
-    gmx_fatal_mpi_va(f_errno, file, line, bMaster, bFinalize, fmt, ap);
+    gmx_fatal_mpi_va(f_errno, file, line, bMain, bFinalize, fmt, ap);
     va_end(ap);
 }
