@@ -35,14 +35,15 @@
 
 #include "grompp.h"
 
-#include <array>
 #include <cerrno>
 #include <climits>
 #include <cmath>
 #include <cstring>
 
 #include <algorithm>
+#include <array>
 #include <memory>
+#include <tuple>
 #include <vector>
 
 #include <sys/types.h>
@@ -69,8 +70,8 @@
 #include "gromacs/gmxpreprocess/toputil.h"
 #include "gromacs/gmxpreprocess/vsite_parm.h"
 #include "gromacs/imd/imd.h"
+#include "gromacs/math/boxmatrix.h"
 #include "gromacs/math/functions.h"
-#include "gromacs/math/invertmatrix.h"
 #include "gromacs/math/units.h"
 #include "gromacs/math/vec.h"
 #include "gromacs/mdlib/calc_verletbuf.h"
@@ -80,6 +81,7 @@
 #include "gromacs/mdlib/perf_est.h"
 #include "gromacs/mdlib/vsite.h"
 #include "gromacs/mdrun/mdmodules.h"
+#include "gromacs/mdrunutility/mdmodulesnotifiers.h"
 #include "gromacs/mdtypes/inputrec.h"
 #include "gromacs/mdtypes/md_enums.h"
 #include "gromacs/mdtypes/nblist.h"
@@ -89,6 +91,7 @@
 #include "gromacs/pulling/pull.h"
 #include "gromacs/random/seed.h"
 #include "gromacs/topology/ifunc.h"
+#include "gromacs/topology/mtop_atomloops.h"
 #include "gromacs/topology/mtop_util.h"
 #include "gromacs/topology/symtab.h"
 #include "gromacs/topology/topology.h"
@@ -104,7 +107,6 @@
 #include "gromacs/utility/listoflists.h"
 #include "gromacs/utility/logger.h"
 #include "gromacs/utility/loggerbuilder.h"
-#include "gromacs/utility/mdmodulesnotifiers.h"
 #include "gromacs/utility/smalloc.h"
 #include "gromacs/utility/snprintf.h"
 
@@ -318,7 +320,7 @@ static int check_atom_names(const char*          fn1,
     return nmismatch;
 }
 
-static void check_bonds_timestep(const gmx_mtop_t* mtop, double dt, warninp* wi)
+static void check_bonds_timestep(const gmx_mtop_t* mtop, double dt, WarningHandler* wi)
 {
     /* This check is not intended to ensure accurate integration,
      * rather it is to signal mistakes in the mdp settings.
@@ -449,11 +451,11 @@ static void check_bonds_timestep(const gmx_mtop_t* mtop, double dt, warninp* wi)
                        : "Maybe you forgot to change the constraints mdp option.");
         if (bWarn)
         {
-            warning(wi, warningMessage.c_str());
+            wi->addWarning(warningMessage);
         }
         else
         {
-            warning_note(wi, warningMessage.c_str());
+            wi->addNote(warningMessage);
         }
     }
 }
@@ -472,7 +474,7 @@ static void check_vel(gmx_mtop_t* mtop, rvec v[])
     }
 }
 
-static void check_shells_inputrec(gmx_mtop_t* mtop, t_inputrec* ir, warninp* wi)
+static void check_shells_inputrec(gmx_mtop_t* mtop, t_inputrec* ir, WarningHandler* wi)
 {
     int nshells = 0;
 
@@ -486,11 +488,11 @@ static void check_shells_inputrec(gmx_mtop_t* mtop, t_inputrec* ir, warninp* wi)
     }
     if ((nshells > 0) && (ir->nstcalcenergy != 1))
     {
-        set_warning_line(wi, "unknown", -1);
+        wi->setFileAndLineNumber("unknown", -1);
         std::string warningMessage = gmx::formatString(
                 "There are %d shells, changing nstcalcenergy from %d to 1", nshells, ir->nstcalcenergy);
         ir->nstcalcenergy = 1;
-        warning(wi, warningMessage.c_str());
+        wi->addWarning(warningMessage);
     }
 }
 
@@ -587,7 +589,7 @@ static void new_status(const char*                           topfile,
                        double*                               reppow,
                        real*                                 fudgeQQ,
                        gmx_bool                              bMorse,
-                       warninp*                              wi,
+                       WarningHandler*                       wi,
                        const gmx::MDLogger&                  logger)
 {
     std::vector<gmx_molblock_t> molblock;
@@ -648,10 +650,10 @@ static void new_status(const char*                           topfile,
         i = rm_interactions(F_DISRES, *mi);
         if (i > 0)
         {
-            set_warning_line(wi, "unknown", -1);
+            wi->setFileAndLineNumber("unknown", -1);
             std::string warningMessage =
                     gmx::formatString("disre = no, removed %d distance restraints", i);
-            warning_note(wi, warningMessage.c_str());
+            wi->addNote(warningMessage);
         }
     }
     if (!opts->bOrire)
@@ -659,10 +661,10 @@ static void new_status(const char*                           topfile,
         i = rm_interactions(F_ORIRES, *mi);
         if (i > 0)
         {
-            set_warning_line(wi, "unknown", -1);
+            wi->setFileAndLineNumber("unknown", -1);
             std::string warningMessage =
                     gmx::formatString("orire = no, removed %d orientation restraints", i);
-            warning_note(wi, warningMessage.c_str());
+            wi->addWarning(warningMessage);
         }
     }
 
@@ -678,12 +680,12 @@ static void new_status(const char*                           topfile,
     if (opts->nshake == eshALLBONDS && ffParametrizedWithHBondConstraints && ir->delta_t < 0.0026
         && gmx_mtop_ftype_count(*sys, F_VSITE3FD) == 0)
     {
-        set_warning_line(wi, "unknown", -1);
-        warning_note(wi,
-                     "You are using constraints on all bonds, whereas the forcefield "
-                     "has been parametrized only with constraints involving hydrogen atoms. "
-                     "We suggest using constraints = h-bonds instead, this will also improve "
-                     "performance.");
+        wi->setFileAndLineNumber("unknown", -1);
+        wi->addNote(
+                "You are using constraints on all bonds, whereas the forcefield "
+                "has been parametrized only with constraints involving hydrogen atoms. "
+                "We suggest using constraints = h-bonds instead, this will also improve "
+                "performance.");
     }
 
     /* COORDINATE file processing */
@@ -696,7 +698,8 @@ static void new_status(const char*                           topfile,
     rvec*       x = nullptr;
     rvec*       v = nullptr;
     snew(conftop, 1);
-    read_tps_conf(confin, conftop, nullptr, &x, &v, state->box, FALSE);
+    // Note that all components in v are set to zero when no v is present in confin
+    read_tps_conf(confin, conftop, nullptr, &x, EI_DYNAMICS(ir->eI) ? &v : nullptr, state->box, FALSE);
     state->natoms = conftop->atoms.nr;
     if (state->natoms != sys->natoms)
     {
@@ -712,15 +715,16 @@ static void new_status(const char*                           topfile,
      * a priori if the number of atoms in confin matches what we expect.
      */
     state->flags |= enumValueToBitMask(StateEntry::X);
-    if (v != nullptr)
+    if (EI_DYNAMICS(ir->eI))
     {
         state->flags |= enumValueToBitMask(StateEntry::V);
     }
     state_change_natoms(state, state->natoms);
     std::copy(x, x + state->natoms, state->x.data());
     sfree(x);
-    if (v != nullptr)
+    if (EI_DYNAMICS(ir->eI))
     {
+        GMX_RELEASE_ASSERT(v, "With dynamics we expect a velocity vector");
         std::copy(v, v + state->natoms, state->v.data());
         sfree(v);
     }
@@ -741,7 +745,7 @@ static void new_status(const char*                           topfile,
                 (nmismatch == 1) ? "" : "s",
                 topfile,
                 confin);
-        warning(wi, warningMessage.c_str());
+        wi->addWarning(warningMessage);
     }
 
     /* Do more checks, mostly related to constraints */
@@ -760,9 +764,8 @@ static void new_status(const char*                           topfile,
 
     if (bGenVel)
     {
-        real* mass;
+        std::vector<real> mass(state->natoms);
 
-        snew(mass, state->natoms);
         for (const AtomProxy atomP : AtomRange(*sys))
         {
             const t_atom& local = atomP.atom();
@@ -774,11 +777,11 @@ static void new_status(const char*                           topfile,
         {
             GMX_LOG(logger.info).asParagraph().appendTextFormatted("Setting gen_seed to %d", opts->seed);
         }
-        state->flags |= enumValueToBitMask(StateEntry::V);
+        GMX_RELEASE_ASSERT((state->flags | enumValueToBitMask(StateEntry::V)) != 0,
+                           "Generate velocities only makes sense when they are used");
         maxwell_speed(opts->tempi, opts->seed, sys, state->v.rvec_array(), logger);
 
-        stop_cm(logger, state->natoms, mass, state->x.rvec_array(), state->v.rvec_array());
-        sfree(mass);
+        stop_cm(logger, state->natoms, mass.data(), state->x.rvec_array(), state->v.rvec_array());
     }
 }
 
@@ -903,10 +906,11 @@ static void cont_status(const char*             slog,
     GMX_LOG(logger.info).asParagraph().appendTextFormatted("Using frame at t = %g ps", use_time);
     GMX_LOG(logger.info).asParagraph().appendTextFormatted("Starting time for run is %g ps", ir->init_t);
 
-    if ((ir->epc != PressureCoupling::No || ir->etc == TemperatureCoupling::NoseHoover) && ener)
+    if ((ir->pressureCouplingOptions.epc != PressureCoupling::No || ir->etc == TemperatureCoupling::NoseHoover)
+        && ener)
     {
         get_enx_state(ener, use_time, sys->groups, ir, state);
-        preserve_box_shape(ir, state->box_rel, state->boxv);
+        preserveBoxShape(ir->pressureCouplingOptions, ir->deform, state->box_rel, state->boxv);
     }
 }
 
@@ -917,7 +921,7 @@ static void read_posres(gmx_mtop_t*                              mtop,
                         RefCoordScaling                          rc_scaling,
                         PbcType                                  pbcType,
                         rvec                                     com,
-                        warninp*                                 wi,
+                        WarningHandler*                          wi,
                         const gmx::MDLogger&                     logger)
 {
     gmx_bool*   hadAtom;
@@ -945,7 +949,7 @@ static void read_posres(gmx_mtop_t*                              mtop,
                 mtop->natoms,
                 std::min(mtop->natoms, natoms),
                 fn);
-        warning(wi, warningMessage.c_str());
+        wi->addWarning(warningMessage);
     }
 
     npbcdim = numPbcDimensions(pbcType);
@@ -1121,7 +1125,7 @@ static void gen_posres(gmx_mtop_t*                              mtop,
                        PbcType                                  pbcType,
                        rvec                                     com,
                        rvec                                     comB,
-                       warninp*                                 wi,
+                       WarningHandler*                          wi,
                        const gmx::MDLogger&                     logger)
 {
     read_posres(mtop, mi, FALSE, fnA, rc_scaling, pbcType, com, wi, logger);
@@ -1134,7 +1138,7 @@ static void gen_posres(gmx_mtop_t*                              mtop,
 static void set_wall_atomtype(PreprocessingAtomTypes* at,
                               t_gromppopts*           opts,
                               t_inputrec*             ir,
-                              warninp*                wi,
+                              WarningHandler*         wi,
                               const gmx::MDLogger&    logger)
 {
     int i;
@@ -1150,7 +1154,7 @@ static void set_wall_atomtype(PreprocessingAtomTypes* at,
         {
             std::string warningMessage = gmx::formatString(
                     "Specified wall atom type %s is not defined", opts->wall_atomtype[i]);
-            warning_error(wi, warningMessage.c_str());
+            wi->addError(warningMessage);
         }
         else
         {
@@ -1324,7 +1328,9 @@ static void init_cmap_grid(gmx_cmap_t* cmap_grid, int ngrid, int grid_spacing)
 }
 
 
-static int count_constraints(const gmx_mtop_t* mtop, gmx::ArrayRef<const MoleculeInformation> mi, warninp* wi)
+static int count_constraints(const gmx_mtop_t*                        mtop,
+                             gmx::ArrayRef<const MoleculeInformation> mi,
+                             WarningHandler*                          wi)
 {
     int count, count_mol;
 
@@ -1355,7 +1361,7 @@ static int count_constraints(const gmx_mtop_t* mtop, gmx::ArrayRef<const Molecul
                     *mi[molb.type].name,
                     count_mol,
                     nrdf_internal(&mi[molb.type].atoms));
-            warning(wi, warningMessage.c_str());
+            wi->addWarning(warningMessage);
         }
         count += molb.nmol * count_mol;
     }
@@ -1382,7 +1388,7 @@ static real calc_temp(const gmx_mtop_t* mtop, const t_inputrec* ir, rvec* v)
     return sum_mv2 / (nrdf * gmx::c_boltz);
 }
 
-static real get_max_reference_temp(const t_inputrec* ir, warninp* wi)
+static real get_max_reference_temp(const t_inputrec* ir, WarningHandler* wi)
 {
     real ref_t;
     int  i;
@@ -1409,7 +1415,7 @@ static real get_max_reference_temp(const t_inputrec* ir, warninp* wi)
                 "their temperature is not more than %.3f K. If their temperature is higher, the "
                 "energy error and the Verlet buffer might be underestimated.",
                 ref_t);
-        warning(wi, warningMessage.c_str());
+        wi->addWarning(warningMessage);
     }
 
     return ref_t;
@@ -1418,7 +1424,10 @@ static real get_max_reference_temp(const t_inputrec* ir, warninp* wi)
 /* Checks if there are unbound atoms in moleculetype molt.
  * Prints a note for each unbound atoms and a warning if any is present.
  */
-static void checkForUnboundAtoms(const gmx_moltype_t* molt, gmx_bool bVerbose, warninp* wi, const gmx::MDLogger& logger)
+static void checkForUnboundAtoms(const gmx_moltype_t* molt,
+                                 gmx_bool             bVerbose,
+                                 WarningHandler*      wi,
+                                 const gmx::MDLogger& logger)
 {
     const t_atoms* atoms = &molt->atoms;
 
@@ -1478,12 +1487,15 @@ static void checkForUnboundAtoms(const gmx_moltype_t* molt, gmx_bool bVerbose, w
                 "definition by mistake. Run with -v to get information for each atom.",
                 *molt->name,
                 numDanglingAtoms);
-        warning_note(wi, warningMessage.c_str());
+        wi->addNote(warningMessage);
     }
 }
 
 /* Checks all moleculetypes for unbound atoms */
-static void checkForUnboundAtoms(const gmx_mtop_t* mtop, gmx_bool bVerbose, warninp* wi, const gmx::MDLogger& logger)
+static void checkForUnboundAtoms(const gmx_mtop_t*    mtop,
+                                 gmx_bool             bVerbose,
+                                 WarningHandler*      wi,
+                                 const gmx::MDLogger& logger)
 {
     for (const gmx_moltype_t& molt : mtop->moltype)
     {
@@ -1575,7 +1587,7 @@ static bool haveDecoupledModeInMol(const gmx_moltype_t&           molt,
  * When decoupled modes are present and the accuracy in insufficient,
  * this routine issues a warning if the accuracy is insufficient.
  */
-static void checkDecoupledModeAccuracy(const gmx_mtop_t* mtop, const t_inputrec* ir, warninp* wi)
+static void checkDecoupledModeAccuracy(const gmx_mtop_t* mtop, const t_inputrec* ir, WarningHandler* wi)
 {
     /* We only have issues with decoupled modes with normal MD.
      * With stochastic dynamics equipartitioning is enforced strongly.
@@ -1655,16 +1667,17 @@ static void checkDecoupledModeAccuracy(const gmx_mtop_t* mtop, const t_inputrec*
                     "buffer size and thus over the energy drift.",
                     enumValueToString(CutoffScheme::Verlet));
         }
-        warning(wi, message);
+        wi->addWarning(message);
     }
 }
 
-static void set_verlet_buffer(const gmx_mtop_t*    mtop,
-                              t_inputrec*          ir,
-                              real                 buffer_temp,
-                              matrix               box,
-                              warninp*             wi,
-                              const gmx::MDLogger& logger)
+static void set_verlet_buffer(const gmx_mtop_t*              mtop,
+                              t_inputrec*                    ir,
+                              real                           buffer_temp,
+                              gmx::ArrayRef<const gmx::RVec> coordinates,
+                              matrix                         box,
+                              WarningHandler*                wi,
+                              const gmx::MDLogger&           logger)
 {
     GMX_LOG(logger.info)
             .asParagraph()
@@ -1673,17 +1686,20 @@ static void set_verlet_buffer(const gmx_mtop_t*    mtop,
                     ir->verletbuf_tol,
                     buffer_temp);
 
+    const real effectiveAtomDensity = computeEffectiveAtomDensity(
+            coordinates, box, std::max(ir->rcoulomb, ir->rvdw), MPI_COMM_NULL);
+
     /* Calculate the buffer size for simple atom vs atoms list */
     VerletbufListSetup listSetup1x1;
     listSetup1x1.cluster_size_i = 1;
     listSetup1x1.cluster_size_j = 1;
     const real rlist_1x1        = calcVerletBufferSize(
-            *mtop, det(box), *ir, ir->nstlist, ir->nstlist - 1, buffer_temp, listSetup1x1);
+            *mtop, effectiveAtomDensity, *ir, ir->nstlist, ir->nstlist - 1, buffer_temp, listSetup1x1);
 
     /* Set the pair-list buffer size in ir */
     VerletbufListSetup listSetup4x4 = verletbufGetSafeListSetup(ListSetupType::CpuNoSimd);
     ir->rlist                       = calcVerletBufferSize(
-            *mtop, det(box), *ir, ir->nstlist, ir->nstlist - 1, buffer_temp, listSetup4x4);
+            *mtop, effectiveAtomDensity, *ir, ir->nstlist, ir->nstlist - 1, buffer_temp, listSetup4x4);
 
     const int n_nonlin_vsite = gmx::countNonlinearVsites(*mtop);
     if (n_nonlin_vsite > 0)
@@ -1693,7 +1709,7 @@ static void set_verlet_buffer(const gmx_mtop_t*    mtop,
                 "energy error is approximated. In most cases this does not affect the error "
                 "significantly.",
                 n_nonlin_vsite);
-        warning_note(wi, warningMessage);
+        wi->addNote(warningMessage);
     }
 
     GMX_LOG(logger.info)
@@ -1731,17 +1747,19 @@ static void set_verlet_buffer(const gmx_mtop_t*    mtop,
 }
 
 // Computes and returns that largest distance between non-perturbed excluded atom pairs
-static real maxNonPerturbedExclusionDistance(const gmx_mtop_t&              mtop,
-                                             const bool                     useFep,
-                                             const PbcType                  pbcType,
-                                             gmx::ArrayRef<const gmx::RVec> x,
-                                             const matrix                   box)
+static std::tuple<real, int, int> maxNonPerturbedExclusionDistance(const gmx_mtop_t& mtop,
+                                                                   const bool        useFep,
+                                                                   const PbcType     pbcType,
+                                                                   gmx::ArrayRef<const gmx::RVec> x,
+                                                                   const matrix box)
 {
     t_pbc pbc;
 
     set_pbc(&pbc, pbcType, box);
 
     real dx2Max = 0;
+    int  atom0  = -1;
+    int  atom1  = -1;
 
     int moleculeOffset = 0;
     for (const auto& mb : mtop.molblock)
@@ -1769,7 +1787,13 @@ static real maxNonPerturbedExclusionDistance(const gmx_mtop_t&              mtop
                         rvec dx;
 
                         pbc_dx(&pbc, x[moleculeOffset + iAtom], x[moleculeOffset + jAtom], dx);
-                        dx2Max = std::max(dx2Max, norm2(dx));
+                        const real distanceSquared = norm2(dx);
+                        if (distanceSquared > dx2Max)
+                        {
+                            dx2Max = distanceSquared;
+                            atom0  = moleculeOffset + iAtom;
+                            atom1  = moleculeOffset + jAtom;
+                        }
                     }
                 }
             }
@@ -1778,7 +1802,7 @@ static real maxNonPerturbedExclusionDistance(const gmx_mtop_t&              mtop
         }
     }
 
-    return std::sqrt(dx2Max);
+    return std::make_tuple(std::sqrt(dx2Max), atom0, atom1);
 }
 
 // Computes and logs the maximum exclusion distance. Checks whether (non-perturbed) excluded
@@ -1788,18 +1812,21 @@ static void checkExclusionDistances(const gmx_mtop_t&              mtop,
                                     gmx::ArrayRef<const gmx::RVec> x,
                                     const matrix                   box,
                                     const gmx::MDLogger&           logger,
-                                    warninp*                       wi)
+                                    WarningHandler*                wi)
 {
     // Check the maximum distance for (non-perturbed) excluded pairs here,
     // as it should not be longer than the cut-off distance, but we can't
     // easily ensure that during the run.
-    const bool useFep               = (ir.efep != FreeEnergyPerturbationType::No);
-    const real maxExclusionDistance = maxNonPerturbedExclusionDistance(mtop, useFep, ir.pbcType, x, box);
+    const bool useFep = (ir.efep != FreeEnergyPerturbationType::No);
+    const auto [maxExclusionDistance, atom0, atom1] =
+            maxNonPerturbedExclusionDistance(mtop, useFep, ir.pbcType, x, box);
 
-    const std::string distanceString =
-            gmx::formatString("The largest distance between%s excluded atoms is %.3f nm",
-                              useFep ? " non-perturbed" : "",
-                              maxExclusionDistance);
+    const std::string distanceString = gmx::formatString(
+            "The largest distance between%s excluded atoms is %.3f nm between atom %d and %d",
+            useFep ? " non-perturbed" : "",
+            maxExclusionDistance,
+            atom0 + 1,
+            atom1 + 1);
 
     GMX_LOG(logger.info).asParagraph().appendText(distanceString);
 
@@ -1814,11 +1841,11 @@ static void checkExclusionDistances(const gmx_mtop_t&              mtop,
         {
             text += " If you expect that minimization will bring such distances within the "
                     "cut-off, you can ignore this warning.";
-            warning(wi, text.c_str());
+            wi->addWarning(text);
         }
         else
         {
-            warning_error(wi, text.c_str());
+            wi->addError(text);
         }
     }
     else if (maxExclusionDistance > 0.9_real * cutoffDistance)
@@ -1830,11 +1857,11 @@ static void checkExclusionDistances(const gmx_mtop_t&              mtop,
                 distanceString.c_str());
         if (EI_DYNAMICS(ir.eI))
         {
-            warning(wi, text.c_str());
+            wi->addWarning(text);
         }
         else
         {
-            warning_note(wi, text.c_str());
+            wi->addNote(text);
         }
     }
 }
@@ -1947,7 +1974,6 @@ int gmx_grompp(int argc, char* argv[])
     bool                                 bNeedVel, bGenVel;
     gmx_output_env_t*                    oenv;
     gmx_bool                             bVerbose = FALSE;
-    warninp*                             wi;
 
     t_filenm fnm[] = { { efMDP, nullptr, nullptr, ffREAD },
                        { efMDP, "-po", "mdout", ffWRITE },
@@ -2021,14 +2047,14 @@ int gmx_grompp(int argc, char* argv[])
     const gmx::MDLogger logger(logOwner.logger());
 
 
-    wi = init_warning(TRUE, maxwarn);
+    WarningHandler wi{ true, maxwarn };
 
     /* PARAMETER file processing */
     mdparin = opt2fn("-f", NFILE, fnm);
-    set_warning_line(wi, mdparin, -1);
+    wi.setFileAndLineNumber(mdparin, -1);
     try
     {
-        get_ir(mdparin, opt2fn("-po", NFILE, fnm), &mdModules, ir, opts, WriteMdpHeader::yes, wi);
+        get_ir(mdparin, opt2fn("-po", NFILE, fnm), &mdModules, ir, opts, WriteMdpHeader::yes, &wi);
     }
     GMX_CATCH_ALL_AND_EXIT_WITH_FATAL_ERROR
 
@@ -2040,7 +2066,7 @@ int gmx_grompp(int argc, char* argv[])
     mdModules.notifiers().preProcessingNotifier_.notify(logger);
 
     // Notify MDModules of existing warninp
-    mdModules.notifiers().preProcessingNotifier_.notify(wi);
+    mdModules.notifiers().preProcessingNotifier_.notify(&wi);
 
     // Notify QMMM MDModule of external QM input file command-line option
     {
@@ -2054,7 +2080,7 @@ int gmx_grompp(int argc, char* argv[])
                 .asParagraph()
                 .appendTextFormatted("checking input for internal consistency...");
     }
-    check_ir(mdparin, mdModules.notifiers(), ir, opts, wi);
+    check_ir(mdparin, mdModules.notifiers(), ir, opts, &wi);
 
     if (ir->ld_seed == -1)
     {
@@ -2080,7 +2106,7 @@ int gmx_grompp(int argc, char* argv[])
                 "Generating velocities is inconsistent with attempting "
                 "to continue a previous run. Choose only one of "
                 "gen-vel = yes and continuation = yes.");
-        warning_error(wi, warningMessage);
+        wi.addError(warningMessage);
     }
 
     std::array<InteractionsOfType, F_NRE> interactions;
@@ -2116,7 +2142,7 @@ int gmx_grompp(int argc, char* argv[])
                &reppow,
                &fudgeQQ,
                opts->bMorse,
-               wi,
+               &wi,
                logger);
 
     if (debug)
@@ -2140,7 +2166,7 @@ int gmx_grompp(int argc, char* argv[])
         }
     }
 
-    if ((count_constraints(&sys, mi, wi) != 0) && (ir->eConstrAlg == ConstraintAlgorithm::Shake))
+    if ((count_constraints(&sys, mi, &wi) != 0) && (ir->eConstrAlg == ConstraintAlgorithm::Shake))
     {
         if (ir->eI == IntegrationAlgorithm::CG || ir->eI == IntegrationAlgorithm::LBFGS)
         {
@@ -2149,7 +2175,7 @@ int gmx_grompp(int argc, char* argv[])
                                       enumValueToString(ir->eI),
                                       enumValueToString(ConstraintAlgorithm::Shake),
                                       enumValueToString(ConstraintAlgorithm::Lincs));
-            warning_error(wi, warningMessage);
+            wi.addError(warningMessage);
         }
         if (ir->bPeriodicMols)
         {
@@ -2157,13 +2183,13 @@ int gmx_grompp(int argc, char* argv[])
                     gmx::formatString("Can not do periodic molecules with %s, use %s",
                                       enumValueToString(ConstraintAlgorithm::Shake),
                                       enumValueToString(ConstraintAlgorithm::Lincs));
-            warning_error(wi, warningMessage);
+            wi.addError(warningMessage);
         }
     }
 
     if (EI_SD(ir->eI) && ir->etc != TemperatureCoupling::No)
     {
-        warning_note(wi, "Temperature coupling is ignored with SD integrators.");
+        wi.addNote("Temperature coupling is ignored with SD integrators.");
     }
 
     /* Check for errors in the input now, since they might cause problems
@@ -2173,15 +2199,16 @@ int gmx_grompp(int argc, char* argv[])
 
     if (nint_ftype(&sys, mi, F_POSRES) > 0 || nint_ftype(&sys, mi, F_FBPOSRES) > 0)
     {
-        if (ir->epc == PressureCoupling::ParrinelloRahman || ir->epc == PressureCoupling::Mttk)
+        if (ir->pressureCouplingOptions.epc == PressureCoupling::ParrinelloRahman
+            || ir->pressureCouplingOptions.epc == PressureCoupling::Mttk)
         {
             std::string warningMessage = gmx::formatString(
                     "You are combining position restraints with %s pressure coupling, which can "
                     "lead to instabilities. If you really want to combine position restraints with "
                     "pressure coupling, we suggest to use %s pressure coupling instead.",
-                    enumValueToString(ir->epc),
+                    enumValueToString(ir->pressureCouplingOptions.epc),
                     enumValueToString(PressureCoupling::Berendsen));
-            warning_note(wi, warningMessage);
+            wi.addNote(warningMessage);
         }
 
         const char* fn = opt2fn("-r", NFILE, fnm);
@@ -2224,7 +2251,16 @@ int gmx_grompp(int argc, char* argv[])
             }
             GMX_LOG(logger.info).asParagraph().appendText(message);
         }
-        gen_posres(&sys, mi, fn, fnB, ir->refcoord_scaling, ir->pbcType, ir->posres_com, ir->posres_comB, wi, logger);
+        gen_posres(&sys,
+                   mi,
+                   fn,
+                   fnB,
+                   ir->pressureCouplingOptions.refcoord_scaling,
+                   ir->pbcType,
+                   ir->posres_com,
+                   ir->posres_comB,
+                   &wi,
+                   logger);
     }
 
     /* If we are using CMAP, setup the pre-interpolation grid */
@@ -2239,7 +2275,7 @@ int gmx_grompp(int argc, char* argv[])
                    &sys.ffparams.cmap_grid);
     }
 
-    set_wall_atomtype(&atypes, opts, ir, wi, logger);
+    set_wall_atomtype(&atypes, opts, ir, &wi, logger);
     if (bRenum)
     {
         atypes.renumberTypes(interactions, &sys, ir->wall_atomtype, bVerbose);
@@ -2250,9 +2286,6 @@ int gmx_grompp(int argc, char* argv[])
         gmx_fatal(FARGS, "Implicit solvation is no longer supported");
     }
 
-    /* PELA: Copy the atomtype data to the topology atomtype list */
-    atypes.copyTot_atomtypes(&(sys.atomtypes));
-
     if (debug)
     {
         pr_symtab(debug, 0, "After atype.renumberTypes", &sys.symtab);
@@ -2288,31 +2321,30 @@ int gmx_grompp(int argc, char* argv[])
     }
 
     /* check for shells and inpurecs */
-    check_shells_inputrec(&sys, ir, wi);
+    check_shells_inputrec(&sys, ir, &wi);
 
     /* check masses */
-    check_mol(&sys, wi);
+    check_mol(&sys, &wi);
 
     if (haveFepPerturbedMassesInSettles(sys))
     {
-        warning_error(wi,
-                      "SETTLE is not implemented for atoms whose mass is perturbed. "
-                      "You might instead use normal constraints.");
+        wi.addError(
+                "SETTLE is not implemented for atoms whose mass is perturbed. "
+                "You might instead use normal constraints.");
     }
 
-    checkForUnboundAtoms(&sys, bVerbose, wi, logger);
+    checkForUnboundAtoms(&sys, bVerbose, &wi, logger);
 
     if (EI_DYNAMICS(ir->eI) && ir->eI != IntegrationAlgorithm::BD)
     {
-        check_bonds_timestep(&sys, ir->delta_t, wi);
+        check_bonds_timestep(&sys, ir->delta_t, &wi);
     }
 
-    checkDecoupledModeAccuracy(&sys, ir, wi);
+    checkDecoupledModeAccuracy(&sys, ir, &wi);
 
     if (EI_ENERGY_MINIMIZATION(ir->eI) && 0 == ir->nsteps)
     {
-        warning_note(
-                wi,
+        wi.addNote(
                 "Zero-step energy minimization will alter the coordinates before calculating the "
                 "energy. If you just want the energy of a single point, try zero-step MD (with "
                 "unconstrained_start = yes). To do multiple single-point energy evaluations of "
@@ -2325,15 +2357,15 @@ int gmx_grompp(int argc, char* argv[])
     {
         GMX_LOG(logger.info).asParagraph().appendTextFormatted("initialising group options...");
     }
-    do_index(mdparin, ftp2fn_null(efNDX, NFILE, fnm), &sys, bVerbose, mdModules.notifiers(), ir, wi);
+    do_index(mdparin, ftp2fn_null(efNDX, NFILE, fnm), &sys, bVerbose, mdModules.notifiers(), ir, &wi);
 
     // Notify topology to MdModules for pre-processing after all indexes were built
     mdModules.notifiers().preProcessingNotifier_.notify(&sys);
 
-    if (EEL_FULL(ir->coulombtype) || EVDW_PME(ir->vdwtype))
+    if (usingFullElectrostatics(ir->coulombtype) || usingLJPme(ir->vdwtype))
     {
         // We may have exclusion forces beyond the cut-off distance.
-        checkExclusionDistances(sys, *ir, state.x, state.box, logger, wi);
+        checkExclusionDistances(sys, *ir, state.x, state.box, logger, &wi);
     }
 
     if (ir->cutoff_scheme == CutoffScheme::Verlet && ir->verletbuf_tol > 0)
@@ -2358,7 +2390,7 @@ int gmx_grompp(int argc, char* argv[])
                             "NVE simulation: will use the initial temperature of %.3f K for "
                             "determining the Verlet buffer size",
                             buffer_temp);
-                    warning_note(wi, warningMessage);
+                    wi.addNote(warningMessage);
                 }
                 else
                 {
@@ -2366,12 +2398,12 @@ int gmx_grompp(int argc, char* argv[])
                             "NVE simulation with an initial temperature of zero: will use a Verlet "
                             "buffer of %d%%. Check your energy drift!",
                             gmx::roundToInt(verlet_buffer_ratio_NVE_T0 * 100));
-                    warning_note(wi, warningMessage);
+                    wi.addNote(warningMessage);
                 }
             }
             else
             {
-                buffer_temp = get_max_reference_temp(ir, wi);
+                buffer_temp = get_max_reference_temp(ir, &wi);
             }
 
             if (EI_MD(ir->eI) && ir->etc == TemperatureCoupling::No && buffer_temp == 0)
@@ -2411,11 +2443,11 @@ int gmx_grompp(int argc, char* argv[])
                                 gmx::roundToInt(ir->verletbuf_tol / totalEnergyDriftPerAtomPerPicosecond * 100),
                                 gmx::roundToInt(100 * driftTolerance),
                                 driftTolerance * totalEnergyDriftPerAtomPerPicosecond);
-                        warning_note(wi, warningMessage);
+                        wi.addNote(warningMessage);
                     }
                 }
 
-                set_verlet_buffer(&sys, ir, buffer_temp, state.box, wi, logger);
+                set_verlet_buffer(&sys, ir, buffer_temp, state.x, state.box, &wi, logger);
             }
         }
     }
@@ -2428,7 +2460,7 @@ int gmx_grompp(int argc, char* argv[])
         pr_symtab(debug, 0, "After index", &sys.symtab);
     }
 
-    triple_check(mdparin, ir, &sys, wi);
+    triple_check(mdparin, ir, &sys, &wi);
     close_symtab(&sys.symtab);
     if (debug)
     {
@@ -2465,7 +2497,7 @@ int gmx_grompp(int argc, char* argv[])
         clear_rvec(state.box[ZZ]);
     }
 
-    if (EEL_FULL(ir->coulombtype) || EVDW_PME(ir->vdwtype))
+    if (usingFullElectrostatics(ir->coulombtype) || usingLJPme(ir->vdwtype))
     {
         /* Calculate the optimal grid dimensions */
         matrix          scaledBox;
@@ -2479,17 +2511,16 @@ int gmx_grompp(int argc, char* argv[])
         }
         else if (ir->nkx != 0 && ir->nky != 0 && ir->nkz != 0)
         {
-            set_warning_line(wi, mdparin, -1);
-            warning_error(
-                    wi, "Some of the Fourier grid sizes are set, but all of them need to be set.");
+            wi.setFileAndLineNumber(mdparin, -1);
+            wi.addError("Some of the Fourier grid sizes are set, but all of them need to be set.");
         }
         const int minGridSize = minimalPmeGridSize(ir->pme_order);
         calcFftGrid(stdout, scaledBox, ir->fourier_spacing, minGridSize, &(ir->nkx), &(ir->nky), &(ir->nkz));
         if (ir->nkx < minGridSize || ir->nky < minGridSize || ir->nkz < minGridSize)
         {
-            warning_error(wi,
-                          "The PME grid size should be >= 2*(pme-order - 1); either manually "
-                          "increase the grid size or decrease pme-order");
+            wi.addError(
+                    "The PME grid size should be >= 2*(pme-order - 1); either manually "
+                    "increase the grid size or decrease pme-order");
         }
     }
 
@@ -2525,7 +2556,7 @@ int gmx_grompp(int argc, char* argv[])
     if (ir->bPull)
     {
         pull = set_pull_init(
-                ir, sys, state.x, state.box, state.lambda[FreeEnergyPerturbationCouplingType::Mass], wi);
+                ir, sys, state.x, state.box, state.lambda[FreeEnergyPerturbationCouplingType::Mass], &wi);
     }
 
     /* Modules that supply external potential for pull coordinates
@@ -2536,9 +2567,9 @@ int gmx_grompp(int argc, char* argv[])
     if (ir->bDoAwh)
     {
         tensor compressibility = { { 0 } };
-        if (ir->epc != PressureCoupling::No)
+        if (ir->pressureCouplingOptions.epc != PressureCoupling::No)
         {
-            copy_mat(ir->compress, compressibility);
+            copy_mat(ir->pressureCouplingOptions.compress, compressibility);
         }
         setStateDependentAwhParams(
                 ir->awhParams.get(),
@@ -2547,12 +2578,12 @@ int gmx_grompp(int argc, char* argv[])
                 state.box,
                 ir->pbcType,
                 compressibility,
-                &ir->opts,
+                *ir,
                 ir->efep != FreeEnergyPerturbationType::No ? ir->fepvals->all_lambda[static_cast<int>(
                         FreeEnergyPerturbationCouplingType::Fep)][ir->fepvals->init_fep_state]
                                                            : 0,
                 sys,
-                wi);
+                &wi);
     }
 
     if (ir->bPull)
@@ -2562,17 +2593,17 @@ int gmx_grompp(int argc, char* argv[])
 
     if (ir->bRot)
     {
-        set_reference_positions(ir->rot,
+        set_reference_positions(ir->rot.get(),
                                 state.x.rvec_array(),
                                 state.box,
                                 opt2fn("-ref", NFILE, fnm),
                                 opt2bSet("-ref", NFILE, fnm),
-                                wi);
+                                &wi);
     }
 
     /*  reset_multinr(sys); */
 
-    if (EEL_PME(ir->coulombtype))
+    if (usingPme(ir->coulombtype))
     {
         float ratio = pme_load_estimate(sys, *ir, state.box);
         GMX_LOG(logger.info)
@@ -2586,16 +2617,15 @@ int gmx_grompp(int argc, char* argv[])
         if ((ir->efep == FreeEnergyPerturbationType::No && ratio > 1.0 / 2.0)
             || (ir->efep != FreeEnergyPerturbationType::No && ratio > 2.0 / 3.0))
         {
-            warning_note(
-                    wi,
+            wi.addNote(
                     "The optimal PME mesh load for parallel simulations is below 0.5\n"
                     "and for highly parallel simulations between 0.25 and 0.33,\n"
                     "for higher performance, increase the cut-off and the PME grid spacing.\n");
             if (ir->efep != FreeEnergyPerturbationType::No)
             {
-                warning_note(wi,
-                             "For free energy simulations, the optimal load limit increases from "
-                             "0.5 to 0.667\n");
+                wi.addNote(
+                        "For free energy simulations, the optimal load limit increases from "
+                        "0.5 to 0.667\n");
             }
         }
     }
@@ -2606,8 +2636,8 @@ int gmx_grompp(int argc, char* argv[])
                 gmx::formatString("This run will generate roughly %.0f Mb of data", cio);
         if (cio > 2000)
         {
-            set_warning_line(wi, mdparin, -1);
-            warning_note(wi, warningMessage);
+            wi.setFileAndLineNumber(mdparin, -1);
+            wi.addNote(warningMessage);
         }
         else
         {
@@ -2639,8 +2669,8 @@ int gmx_grompp(int argc, char* argv[])
         const int nstglobalcomm = computeGlobalCommunicationPeriod(ir);
         if (ir->nstcomm % nstglobalcomm != 0)
         {
-            warning_note(
-                    wi,
+            wi.addNote(
+
                     gmx::formatString(
                             "COM removal frequency is set to (%d).\n"
                             "Other settings require a global communication frequency of %d.\n"
