@@ -33,7 +33,7 @@
  */
 #include "gmxpre.h"
 
-#include "wallcycle.h"
+#include "gromacs/timing/wallcycle.h"
 
 #include "config.h"
 
@@ -41,6 +41,7 @@
 
 #include <array>
 #include <memory>
+#include <string>
 #include <vector>
 
 #include "gromacs/math/functions.h"
@@ -48,9 +49,10 @@
 #include "gromacs/timing/cyclecounter.h"
 #include "gromacs/timing/gpu_timing.h"
 #include "gromacs/timing/wallcyclereporting.h"
-#include "gromacs/utility/arrayref.h"
+#include "gromacs/utility/basedefinitions.h"
 #include "gromacs/utility/cstringutil.h"
 #include "gromacs/utility/enumerationhelpers.h"
+#include "gromacs/utility/fatalerror.h"
 #include "gromacs/utility/gmxassert.h"
 #include "gromacs/utility/gmxmpi.h"
 #include "gromacs/utility/logger.h"
@@ -58,18 +60,26 @@
 #include "gromacs/utility/snprintf.h"
 #include "gromacs/utility/stringutil.h"
 
-//! Whether wallcycle debugging is enabled
-constexpr bool gmx_unused enableWallcycleDebug = (DEBUG_WCYCLE != 0);
-//! True if only the master rank should print debugging output
-constexpr bool gmx_unused onlyMasterDebugPrints = true;
-//! True if cycle counter nesting depth debuggin prints are enabled
-constexpr bool gmx_unused debugPrintDepth = false /* enableWallcycleDebug */;
+//! True if only the main rank should print debugging output
+constexpr bool onlyMainDebugPrints = true;
+//! True if cycle counter nesting depth debugging prints are enabled
+constexpr bool debugPrintDepth = false;
 
-#if DEBUG_WCYCLE
-#    include "gromacs/utility/fatalerror.h"
-#endif
+template<int maxLength, typename Container>
+static constexpr bool checkStringsLengths(const Container& strings)
+{
+    // NOLINTNEXTLINE(readability-use-anyofallof) // std::all_of is constexpr only since C++20
+    for (const char* str : strings)
+    {
+        if (std::char_traits<char>::length(str) > maxLength)
+        {
+            return false;
+        }
+    }
+    return true;
+}
 
-/* Each name should not exceed 19 printing characters
+/* Each name should not exceed 22 printing characters
    (ie. terminating null can be twentieth) */
 static const char* enumValuetoString(WallCycleCounter enumValue)
 {
@@ -83,11 +93,12 @@ static const char* enumValuetoString(WallCycleCounter enumValue)
         "Vsite constr.",
         "Send X to PME",
         "Neighbor search",
-        "Launch GPU ops.",
+        "Launch PP GPU ops.",
         "Comm. coord.",
         "Force",
         "Wait + Comm. F",
         "PME mesh",
+        "PME GPU mesh",
         "PME redist. X/F",
         "PME spread",
         "PME gather",
@@ -95,14 +106,19 @@ static const char* enumValuetoString(WallCycleCounter enumValue)
         "PME 3D-FFT Comm.",
         "PME solve LJ",
         "PME solve Elec",
+        "Wait PME GPU D2H",
+        "PME 3D-FFT",
+        "PME solve",
+        "Wait PME GPU gather",
+        "Reduce GPU PME F",
+        "Launch PME GPU ops.",
+        "Wait PME Recv. PP X",
+        "Wait PME GPU spread",
+        "Wait GPU FFT to PME",
+        "PME Halo exch comm",
         "PME wait for PP",
         "Wait + Recv. PME F",
-        "Wait PME GPU spread",
-        "PME 3D-FFT",
-        "PME solve", /* the strings for FFT/solve are repeated here for mixed mode counters */
-        "Wait PME GPU gather",
         "Wait Bonded GPU",
-        "Reduce GPU PME F",
         "Wait GPU NB nonloc.",
         "Wait GPU NB local",
         "Wait GPU state copy",
@@ -118,11 +134,17 @@ static const char* enumValuetoString(WallCycleCounter enumValue)
         "Add rot. forces",
         "Position swapping",
         "IMD",
+        "MD Graph",
         "Test"
     };
+    static_assert(checkStringsLengths<22>(wallCycleCounterNames));
     return wallCycleCounterNames[enumValue];
 }
 
+// Clang complains about this function not used in builds without subcounters
+// clang-format off
+CLANG_DIAGNOSTIC_IGNORE(-Wunneeded-internal-declaration)
+// clang-format on
 static const char* enumValuetoString(WallCycleSubCounter enumValue)
 {
     constexpr gmx::EnumerationArray<WallCycleSubCounter, const char*> wallCycleSubCounterNames = {
@@ -134,35 +156,43 @@ static const char* enumValuetoString(WallCycleSubCounter enumValue)
         "DD top. other",
         "DD GPU ops.",
         "NS grid local",
-        "NS grid non-loc.",
+        "NS grid non-local",
         "NS search local",
-        "NS search non-loc.",
+        "NS search non-local",
         "Bonded F",
         "Bonded-FEP F",
         "Restraints F",
         "Listed buffer ops.",
-        "Nonbonded pruning",
-        "Nonbonded F kernel",
-        "Nonbonded F clear",
-        "Nonbonded FEP",
-        "Nonbonded FEP reduction",
-        "Launch NB GPU tasks",
-        "Launch Bonded GPU tasks",
-        "Launch PME GPU tasks",
+        "NB pruning",
+        "NB F kernel",
+        "NB F clear",
+        "NB FEP",
+        "NB FEP reduction",
+        "Launch GPU NB tasks",
+        "Launch GPU Bonded",
         "Launch state copy",
         "Ewald F correction",
         "NB X buffer ops.",
         "NB F buffer ops.",
         "Clear force buffer",
-        "Launch GPU NB X buffer ops.",
-        "Launch GPU NB F buffer ops.",
-        "Launch GPU Comm. coord.",
-        "Launch GPU Comm. force.",
+        "Launch GPU NB X ops.",
+        "Launch GPU NB F ops.",
+        "Launch GPU Comm. X",
+        "Launch GPU Comm. F",
         "Launch GPU update",
+        "Launch PME GPU FFT",
+        "Graph wait pre-capture",
+        "Graph capture",
+        "Graph instantiate/upd.",
+        "Graph wait pre-launch",
+        "Graph launch",
+        "Constraints Comm.", // constraints communication time, note that this counter will contain load imbalance
         "Test subcounter"
     };
+    static_assert(checkStringsLengths<22>(wallCycleSubCounterNames));
     return wallCycleSubCounterNames[enumValue];
 }
+CLANG_DIAGNOSTIC_RESET
 
 /* PME GPU timing events' names - correspond to the enum in the gpu_timing.h */
 static const char* enumValuetoString(PmeStage enumValue)
@@ -172,7 +202,7 @@ static const char* enumValuetoString(PmeStage enumValue)
         "PME solve",  "PME 3D-FFT c2r", "PME gather"
     };
     return pmeStageNames[enumValue];
-};
+}
 
 bool wallcycle_have_counter()
 {
@@ -219,57 +249,74 @@ std::unique_ptr<gmx_wallcycle> wallcycle_init(FILE* fplog, int resetstep, const
         wc->wcc_all.resize(sc_numWallCycleCountersSquared);
     }
 
-#if DEBUG_WCYCLE
-    wc->count_depth  = 0;
-    wc->isMasterRank = MASTER(cr);
-#endif
+    // NOLINTNEXTLINE(readability-misleading-indentation)
+    if constexpr (sc_enableWallcycleDebug)
+    {
+        wc->count_depth = 0;
+        wc->isMainRank  = MAIN(cr);
+    }
 
     return wc;
 }
 
-#if DEBUG_WCYCLE
-static void debug_start_check(gmx_wallcycle* wc, WallCycleCounter ewc)
+void debug_start_check(gmx_wallcycle* wc, WallCycleCounter ewc)
 {
-    if (wc->count_depth < 0 || wc->count_depth >= c_MaxWallCycleDepth)
+    // NOLINTNEXTLINE(readability-misleading-indentation)
+    if constexpr (sc_enableWallcycleDebug)
     {
-        gmx_fatal(FARGS, "wallcycle counter depth out of range: %d", wc->count_depth + 1);
-    }
-    wc->counterlist[wc->count_depth] = ewc;
-    wc->count_depth++;
+        // NOLINTNEXTLINE(misc-redundant-expression)
+        if (wc->count_depth < 0 || wc->count_depth >= sc_maxWallCycleDepth)
+        {
+            gmx_fatal(FARGS, "wallcycle counter depth out of range: %d", wc->count_depth + 1);
+        }
+        wc->counterlist[wc->count_depth] = ewc;
+        wc->count_depth++;
 
-    if (debugPrintDepth && (!onlyMasterDebugPrints || wc->isMasterRank))
-    {
-        std::string indentStr(4 * wc->count_depth, ' ');
-        fprintf(stderr, "%swcycle_start depth %d, %s\n", indentStr.c_str(), wc->count_depth, enumValuetoString(ewc));
+        if (debugPrintDepth && (!onlyMainDebugPrints || wc->isMainRank))
+        {
+            std::string indentStr(4 * wc->count_depth, ' ');
+            fprintf(stderr,
+                    "%swcycle_start depth %d, %s\n",
+                    indentStr.c_str(),
+                    wc->count_depth,
+                    enumValuetoString(ewc));
+        }
     }
 }
 
-static void debug_stop_check(gmx_wallcycle* wc, WallCycleCounter ewc)
+void debug_stop_check(gmx_wallcycle* wc, WallCycleCounter ewc)
 {
-    if (debugPrintDepth && (!onlyMasterDebugPrints || wc->isMasterRank))
+    // NOLINTNEXTLINE(readability-misleading-indentation)
+    if constexpr (sc_enableWallcycleDebug)
     {
-        std::string indentStr(4 * wc->count_depth, ' ');
-        fprintf(stderr, "%swcycle_stop  depth %d, %s\n", indentStr.c_str(), wc->count_depth, enumValuetoString(ewc));
-    }
+        if (debugPrintDepth && (!onlyMainDebugPrints || wc->isMainRank))
+        {
+            std::string indentStr(4 * wc->count_depth, ' ');
+            fprintf(stderr,
+                    "%swcycle_stop  depth %d, %s\n",
+                    indentStr.c_str(),
+                    wc->count_depth,
+                    enumValuetoString(ewc));
+        }
 
-    wc->count_depth--;
+        wc->count_depth--;
 
-    if (wc->count_depth < 0)
-    {
-        gmx_fatal(FARGS,
-                  "wallcycle counter depth out of range when stopping %s: %d",
-                  enumValuetoString(ewc),
-                  wc->count_depth);
-    }
-    if (wc->counterlist[wc->count_depth] != ewc)
-    {
-        gmx_fatal(FARGS,
-                  "wallcycle mismatch at stop, start %s, stop %s",
-                  enumValuetoString(wc->counterlist[wc->count_depth]),
-                  enumValuetoString(ewc));
+        if (wc->count_depth < 0)
+        {
+            gmx_fatal(FARGS,
+                      "wallcycle counter depth out of range when stopping %s: %d",
+                      enumValuetoString(ewc),
+                      wc->count_depth);
+        }
+        if (wc->counterlist[wc->count_depth] != ewc)
+        {
+            gmx_fatal(FARGS,
+                      "wallcycle mismatch at stop, start %s, stop %s",
+                      enumValuetoString(wc->counterlist[wc->count_depth]),
+                      enumValuetoString(ewc));
+        }
     }
 }
-#endif
 
 void wallcycle_get(gmx_wallcycle* wc, WallCycleCounter ewc, int* n, double* c)
 {
@@ -279,10 +326,14 @@ void wallcycle_get(gmx_wallcycle* wc, WallCycleCounter ewc, int* n, double* c)
 
 void wallcycle_sub_get(gmx_wallcycle* wc, WallCycleSubCounter ewcs, int* n, double* c)
 {
-    if (sc_useCycleSubcounters && wc != nullptr)
+    // NOLINTNEXTLINE(readability-misleading-indentation)
+    if constexpr (sc_useCycleSubcounters)
     {
-        *n = wc->wcsc[ewcs].n;
-        *c = static_cast<double>(wc->wcsc[ewcs].c);
+        if (wc != nullptr)
+        {
+            *n = wc->wcsc[ewcs].n;
+            *c = static_cast<double>(wc->wcsc[ewcs].c);
+        }
     }
 }
 
@@ -308,10 +359,15 @@ void wallcycle_reset_all(gmx_wallcycle* wc)
             wc->wcc_all[i].c = 0;
         }
     }
-    for (auto& counter : wc->wcsc)
+
+    // NOLINTNEXTLINE(readability-misleading-indentation)
+    if constexpr (sc_useCycleSubcounters)
     {
-        counter.n = 0;
-        counter.c = 0;
+        for (auto& counter : wc->wcsc)
+        {
+            counter.n = 0;
+            counter.c = 0;
+        }
     }
 }
 
@@ -393,11 +449,15 @@ void wallcycle_scale_by_num_threads(gmx_wallcycle* wc, bool isPmeRank, int nthre
             }
         }
     }
-    if (sc_useCycleSubcounters && !isPmeRank)
+    // NOLINTNEXTLINE(readability-misleading-indentation)
+    if constexpr (sc_useCycleSubcounters)
     {
-        for (auto& counter : wc->wcsc)
+        if (!isPmeRank)
         {
-            counter.c *= nthreads_pp;
+            for (auto& counter : wc->wcsc)
+            {
+                counter.c *= nthreads_pp;
+            }
         }
     }
 }
@@ -413,7 +473,7 @@ void wallcycle_scale_by_num_threads(gmx_wallcycle* wc, bool isPmeRank, int nthre
  * uses cycles_sum to manage this, which works OK now because wcsc and
  * wcc_all are unused by the GPU reporting, but it is not satisfactory
  * for the future. Also, there's no need for MPI_Allreduce, since
- * only MASTERRANK uses any of the results. */
+ * only MAINRANK uses any of the results. */
 WallcycleCounts wallcycle_sum(const t_commrec* cr, gmx_wallcycle* wc)
 {
     WallcycleCounts                                    cycles_sum;
@@ -449,12 +509,26 @@ WallcycleCounts wallcycle_sum(const t_commrec* cr, gmx_wallcycle* wc)
         /* The are PME-only nodes */
         if (wcc[WallCycleCounter::PmeMesh].n > 0)
         {
+            GMX_ASSERT(wcc[WallCycleCounter::PmeGpuMesh].c == 0,
+                       "PME mesh GPU ticks should be 0 when PME mesh is running on CPU");
             /* This must be a PME only node, calculate the Wait + Comm. time */
             GMX_ASSERT(wcc[WallCycleCounter::Run].c >= wcc[WallCycleCounter::PmeMesh].c,
                        "Total run ticks must be greater than PME-only ticks");
             wcc[WallCycleCounter::PmeWaitComm].c =
                     wcc[WallCycleCounter::Run].c - wcc[WallCycleCounter::PmeMesh].c;
         }
+
+        if (wcc[WallCycleCounter::PmeGpuMesh].n > 0)
+        {
+            GMX_ASSERT(wcc[WallCycleCounter::PmeMesh].c == 0,
+                       "PME mesh CPU ticks should be 0 when PME mesh is running on GPU");
+
+            /* This must be a PME only node, calculate the Wait + Comm. time */
+            GMX_ASSERT(wcc[WallCycleCounter::Run].c >= wcc[WallCycleCounter::PmeGpuMesh].c,
+                       "Total run ticks must be greater than PME-only ticks");
+            wcc[WallCycleCounter::PmeWaitComm].c =
+                    wcc[WallCycleCounter::Run].c - wcc[WallCycleCounter::PmeGpuMesh].c;
+        }
     }
 
     /* Store the cycles in a double buffer for summing */
@@ -465,7 +539,8 @@ WallcycleCounts wallcycle_sum(const t_commrec* cr, gmx_wallcycle* wc)
 #endif
         cyclesMain[key] = static_cast<double>(wcc[key].c);
     }
-    if (sc_useCycleSubcounters)
+    // NOLINTNEXTLINE(readability-misleading-indentation)
+    if constexpr (sc_useCycleSubcounters)
     {
         for (auto key : keysOf(wc->wcsc))
         {
@@ -488,7 +563,8 @@ WallcycleCounts wallcycle_sum(const t_commrec* cr, gmx_wallcycle* wc)
         double haveInvalidCount = (wc->haveInvalidCount ? 1 : 0);
         // TODO Use MPI_Reduce
         MPI_Allreduce(cyclesMainOnNode.data(), bufMain.data(), bufMain.size(), MPI_DOUBLE, MPI_MAX, cr->mpi_comm_mysim);
-        if (sc_useCycleSubcounters)
+        // NOLINTNEXTLINE(readability-misleading-indentation)
+        if constexpr (sc_useCycleSubcounters)
         {
             MPI_Allreduce(cyclesSubOnNode.data(), bufSub.data(), bufSub.size(), MPI_DOUBLE, MPI_MAX, cr->mpi_comm_mysim);
         }
@@ -498,7 +574,8 @@ WallcycleCounts wallcycle_sum(const t_commrec* cr, gmx_wallcycle* wc)
             wcc[key].n = gmx::roundToInt(bufMain[key]);
         }
         wc->haveInvalidCount = (haveInvalidCount > 0);
-        if (sc_useCycleSubcounters)
+        // NOLINTNEXTLINE(readability-misleading-indentation)
+        if constexpr (sc_useCycleSubcounters)
         {
             for (auto key : keysOf(wc->wcsc))
             {
@@ -508,7 +585,8 @@ WallcycleCounts wallcycle_sum(const t_commrec* cr, gmx_wallcycle* wc)
 
         // TODO Use MPI_Reduce
         MPI_Allreduce(cyclesMain.data(), cycles_sum.data(), cyclesMain.size(), MPI_DOUBLE, MPI_SUM, cr->mpi_comm_mysim);
-        if (sc_useCycleSubcounters)
+        // NOLINTNEXTLINE(readability-misleading-indentation)
+        if constexpr (sc_useCycleSubcounters)
         {
             MPI_Allreduce(cyclesSub.data(),
                           cycles_sum.data() + sc_numWallCycleCounters,
@@ -547,7 +625,8 @@ WallcycleCounts wallcycle_sum(const t_commrec* cr, gmx_wallcycle* wc)
         {
             cycles_sum[static_cast<int>(key)] = cyclesMain[key];
         }
-        if (sc_useCycleSubcounters)
+        // NOLINTNEXTLINE(readability-misleading-indentation)
+        if constexpr (sc_useCycleSubcounters)
         {
             for (auto key : keysOf(cyclesSub))
             {
@@ -601,7 +680,7 @@ print_cycles(FILE* fplog, double c2t, const char* name, int nnodes, int nthreads
         wallt = c_sum * c2t;
 
         fprintf(fplog,
-                " %-19.19s %4s %4s %10s  %10.3f %14.3f %5.1f\n",
+                " %-22.22s %4s %4s %10s  %10.3f %14.3f %5.1f\n",
                 name,
                 nnodes_str,
                 nthreads_str,
@@ -665,8 +744,9 @@ static void print_header(FILE* fplog, int nrank_pp, int nth_pp, int nrank_pme, i
     }
 
     fprintf(fplog, "\n\n");
-    fprintf(fplog, " Computing:          Num   Num      Call    Wall time         Giga-Cycles\n");
-    fprintf(fplog, "                     Ranks Threads  Count      (s)         total sum    %%\n");
+    fprintf(fplog, " Activity:              Num   Num      Call    Wall time         Giga-Cycles\n");
+    fprintf(fplog,
+            "                        Ranks Threads  Count      (s)         total sum    %%\n");
 }
 
 
@@ -687,7 +767,7 @@ void wallcycle_print(FILE*                            fplog,
     int         npp, nth_tot;
     char        buf[STRLEN];
     const char* hline =
-            "-----------------------------------------------------------------------------";
+            "--------------------------------------------------------------------------------";
 
     if (wc == nullptr)
     {
@@ -752,7 +832,7 @@ void wallcycle_print(FILE*                            fplog,
         c2t_pme = 0;
     }
 
-    fprintf(fplog, "\n     R E A L   C Y C L E   A N D   T I M E   A C C O U N T I N G\n\n");
+    fprintf(fplog, "\n      R E A L   C Y C L E   A N D   T I M E   A C C O U N T I N G\n\n");
 
     print_header(fplog, npp, nth_pp, npme, nth_pme);
 
@@ -833,7 +913,7 @@ void wallcycle_print(FILE*                            fplog,
                 hline);
     }
 
-    if (wc->wcc[WallCycleCounter::PmeMesh].n > 0)
+    if (wc->wcc[WallCycleCounter::PmeMesh].n > 0 || wc->wcc[WallCycleCounter::PmeGpuMesh].n > 0)
     {
         // A workaround to not print breakdown when no subcounters were recorded.
         // TODO: figure out and record PME GPU counters (what to do with the waiting ones?)
@@ -850,7 +930,7 @@ void wallcycle_print(FILE*                            fplog,
 
         if (!validPmeSubcounterIndices.empty())
         {
-            fprintf(fplog, " Breakdown of PME mesh computation\n");
+            fprintf(fplog, " Breakdown of PME mesh activities\n");
             fprintf(fplog, "%s\n", hline);
             for (auto i : validPmeSubcounterIndices)
             {
@@ -867,9 +947,10 @@ void wallcycle_print(FILE*                            fplog,
         }
     }
 
-    if (sc_useCycleSubcounters)
+    // NOLINTNEXTLINE(readability-misleading-indentation)
+    if constexpr (sc_useCycleSubcounters)
     {
-        fprintf(fplog, " Breakdown of PP computation\n");
+        fprintf(fplog, " Breakdown of PP / PME activities\n");
         fprintf(fplog, "%s\n", hline);
         for (auto key : keysOf(wc->wcsc))
         {
@@ -994,7 +1075,7 @@ void wallcycle_print(FILE*                            fplog,
              * CPU-GPU load balancing is possible */
             if (gpu_cpu_ratio < 0.8 || gpu_cpu_ratio > 1.25)
             {
-                /* Only the sim master calls this function, so always print to stderr */
+                /* Only the sim main calls this function, so always print to stderr */
                 if (gpu_cpu_ratio < 0.8)
                 {
                     if (npp > 1)
@@ -1051,7 +1132,7 @@ void wallcycle_print(FILE*                            fplog,
         && (cyc_sum[static_cast<int>(WallCycleCounter::Domdec)] > tot * 0.1
             || cyc_sum[static_cast<int>(WallCycleCounter::NS)] > tot * 0.1))
     {
-        /* Only the sim master calls this function, so always print to stderr */
+        /* Only the sim main calls this function, so always print to stderr */
         if (wc->wcc[WallCycleCounter::Domdec].n == 0)
         {
             GMX_LOG(mdlog.warning)
