@@ -487,7 +487,7 @@ int ConvertTpr::run()
 } // namespace
 
 LIBGROMACS_EXPORT const char ConvertTprInfo::name[]             = "convert-tpr";
-LIBGROMACS_EXPORT const char ConvertTprInfo::shortDescription[] = "Make a modifed run-input file";
+LIBGROMACS_EXPORT const char ConvertTprInfo::shortDescription[] = "Make a modified run-input file";
 ICommandLineOptionsModulePointer ConvertTprInfo::create()
 {
     return ICommandLineOptionsModulePointer(std::make_unique<ConvertTpr>());
