@@ -44,7 +44,6 @@
 
 #include <algorithm>
 #include <memory>
-#include <string>
 #include <vector>
 
 #include "gromacs/analysisdata/analysisdata.h"
@@ -52,7 +51,6 @@
 #include "gromacs/analysisdata/modules/histogram.h"
 #include "gromacs/analysisdata/modules/plot.h"
 #include "gromacs/math/units.h"
-#include "gromacs/math/utilities.h"
 #include "gromacs/math/vec.h"
 #include "gromacs/options/basicoptions.h"
 #include "gromacs/options/filenameoption.h"
@@ -62,7 +60,6 @@
 #include "gromacs/selection/selectionoption.h"
 #include "gromacs/trajectory/trajectoryframe.h"
 #include "gromacs/trajectoryanalysis/analysissettings.h"
-#include "gromacs/utility/arrayref.h"
 #include "gromacs/utility/classhelpers.h"
 #include "gromacs/utility/enumerationhelpers.h"
 #include "gromacs/utility/exceptions.h"
@@ -700,7 +697,7 @@ void Angle::analyzeFrame(int frnr, const t_trxframe& fr, t_pbc* pbc, TrajectoryA
 
         // v2 & c2 are conditionally set in the switch statement below, and conditionally
         // used in a different switch statement later. Apparently the clang static analyzer
-        // thinks there are cases where they can be used uninitialzed (which I cannot find),
+        // thinks there are cases where they can be used uninitialized (which I cannot find),
         // but to avoid trouble if we ever change just one of the switch statements it
         // makes sense to clear them outside the first switch.
 
